/* 
 * Proof of concept of: 
 *
 *    Polymorphic shellcodes detection.
 *
 * By: Fermn J. Serna <fjserna@ngsec.com>
 *
 * http://www.ngsec.com
 * Next Generation Security Technologies
 * 
 * NGSEC Research Team <labs@ngsec.com>
 *
 * 22/01/2002
 * 
 */


#include "nidsfindshellcode.h"

struct _conf conf;

int main(int argc, char *argv[]) {
struct _proto proto;
char ch;
int offset,length;
char pcap_err[PCAP_ERRBUF_SIZE];
pcap_t *fdpcap;
u_char *buf;
struct iphdr *ip;
struct tcphdr *tcp;
struct udphdr *udp;
struct icmphdr *icmp;
struct pcap_pkthdr h_pcap;
char *data;


   printf("nidsfindshellcode "VERSION" by Fermn J. Serna <fjserna@ngsec.com>\n");
   printf("Next Generation Security Technologies\n");
   printf("http://www.ngsec.com\n\n");

   memset(&conf,0,sizeof(struct _conf));
   conf.resolve_flag=1;

   while ((ch = getopt(argc, argv, "d:hnpv")) != EOF) {

    switch((char)ch) {
      case 'n':
        conf.resolve_flag=0;
        break;
      case 'd':
        strncpy(conf.device,optarg,DEVICE_LEN-1);
        break;
      case 'h':
        show_help(argv[0]);
        break;
      case 'p':
        conf.promisc_flag=1;
        break;
      case 'v':
        conf.verbose_flag=1;
        break;
      default:
        printf("Error Unknown switch %c\n",ch);
        show_usage(argv[0]);
        break;
      }

   }

 if (strlen(conf.device)==0) 
          show_usage(argv[0]);

 
  if ((fdpcap=pcap_open_live(conf.device,8000,conf.promisc_flag,1000,pcap_err))==NULL){
      printf("Error at pcap_open_live: %s\n",pcap_err);
      exit(1);
      }


  offset=get_datalink_offset(fdpcap);

  while(1) {
   if ((buf=(u_char *)pcap_next(fdpcap,&h_pcap))!=NULL) {

   ip=(struct iphdr *)(buf+offset);

   if ((ip->version)==4) {
    length=(int)(ip->ihl<<2);
    length+=offset;

     memset(&proto,0,sizeof(struct _proto));

     proto.ips=ip->saddr;
     proto.ipd=ip->daddr;

     switch(ip->protocol) {
      case IPPROTO_TCP:
             tcp=(struct tcphdr *)(buf+length);
             data=(char*)(buf+length+(tcp->th_off<<2));
             proto.protocol=IPPROTO_TCP;
             proto.sport=ntohs(tcp->th_sport);
             proto.dport=ntohs(tcp->th_dport);
             check_shellcode(&proto,data,ntohs(ip->tot_len)-(ip->ihl<<2)-(tcp->th_off<<2));
             break;
      case IPPROTO_UDP:
             udp=(struct udphdr *)(buf+length);
             data=(char*)(buf+length+sizeof(struct udphdr));
             proto.protocol=IPPROTO_UDP;
             proto.sport=ntohs(udp->uh_sport);
             proto.dport=ntohs(udp->uh_dport);
             check_shellcode(&proto,data,ntohs(ip->tot_len)-(ip->ihl<<2)-sizeof(struct udphdr));
             break;
      case IPPROTO_ICMP:
             icmp=(struct icmphdr *)(buf+length);
             data=(char *)(buf+length+sizeof(struct icmphdr));
             proto.protocol=IPPROTO_ICMP;
             proto.type=icmp->type;
             proto.code=icmp->code;
             check_shellcode(&proto,data,ntohs(ip->tot_len)-(ip->ihl<<2)-sizeof(struct icmphdr));
             break;
      }
     }
    }  
  } 

  pcap_close(fdpcap);

 return(0);

}


/********************/
/* Show usage stuff */
/********************/
void show_usage(char *proggie) {
 printf("Usage: %s -d <device> [-hnpv]\n",proggie);
 printf("More info %s -h\n",proggie);
 exit(-1);
}

/********************/
/* Show help stuff */
/********************/
void show_help(char *proggie) {
 printf("Usage: %s -d <device> [-hnp]\n",proggie);
 printf("-d <device>\tDevice to sniff\n");
 printf("-n\t\tDon't resolve ips\n");
 printf("-p\t\tPut device in promisc mode\n");
 printf("-h\t\tThis sucking stuff\n");
 printf("-v\t\tVerbose mode\n");
 exit(-1);
}

/*******************************/
/* Returns offset to struct ip */
/*******************************/
int get_datalink_offset(pcap_t *fd) {

  switch (pcap_datalink(fd))
    {
    case DLT_EN10MB:
    case DLT_EN3MB:
      return(14);
      break;
    case DLT_IEEE802:
      return(22);
      break;
    case DLT_PPP:
    case DLT_PPP_BSDOS:
#if (OPENBSD || NETBSD || FREEBSD)
      return(4);
#else
#ifdef SOLARIS
      return(8);
#else
      return(24);
#endif /* SOLARIS */
#endif /* OPENBSD || NETBSD || FREEBSD */
      return(0);
      break;
    case DLT_RAW:
#if (OPENBSD || NETBSD || FREEBSD)
      return(4);
#else
      return(0);
#endif /* OPENBSD || NETBSD || FREEBSD */
      break;
    case DLT_NULL:
      return(4);
      break;
    case DLT_SLIP:
    case DLT_SLIP_BSDOS:
#if (OPENBSD || NETBSD || FREEBSD)
      return(16);
#else
      return(24);
#endif /* OPENBSD || NETBSD || FREEBSD */
      break;
    case DLT_FDDI:
      return(21);
      break;
    default:
      fprintf (stderr, "Error pcap_datalink : Unknown device type!\n");
      exit(-1);
    }
 return(0);

}

/******************/
/* Resolve an ip  */
/* Double check!  */
/******************/
char * resolve(uint32_t ip) {
struct hostent *host, *host2;
struct in_addr addr;

 addr.s_addr=ip;
 if (conf.resolve_flag) {
  if ((host=gethostbyaddr((char*)&ip,sizeof(u_long),AF_INET))==NULL)
        return(inet_ntoa(addr));
   else {
     if ((host2=gethostbyname(host->h_name))==NULL) return(inet_ntoa(addr));
        else {
           for (;host2->h_addr_list[0]!=NULL; host2->h_addr_list++) {
             if (!memcmp(host2->h_addr_list[0], (caddr_t)&addr.s_addr,
                              sizeof(addr.s_addr)))
                     return(host->h_name);
             }
             return(inet_ntoa(addr));
         }
     }
   }
 else return(inet_ntoa(addr));
}
 
